#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/21

"""
import sys
import time

__all__ = [
    'time_start',
    'time_get_seconds',
    'time_check_interval',
    'time_wait_until',
]


def time_start(key='__default__'):
    MoVoidTimer.start(key)


def time_get_seconds(key, digits=2):
    return MoVoidTimer.get_total_seconds(key, digits)


def time_check_interval(interval=1, key='__default__', refresh=True):
    return MoVoidTimer.check_interval(interval, key, refresh)


def time_wait_until(wait_second=1, key='__default__', refresh=True):
    MoVoidTimer.wait_until(wait_second, key, refresh)


def time_backup(to_key, from_key='__default__'):
    MoVoidTimer.backup(to_key, from_key)


class MoVoidTimer:
    time_dict = {
        '__default__': time.time()
    }

    @classmethod
    def start(cls, key='__default__'):
        if key not in cls.time_dict:
            cls.time_dict[key] = 0
        cls.time_dict[key] = time.time()

    @classmethod
    def get_total_seconds(cls, key, digits=2):
        now_time = time.time()
        if key not in cls.time_dict:
            cls.time_dict[key] = now_time
        return round(now_time - cls.time_dict[key], digits)

    @classmethod
    def check_interval(cls, interval=1, key='__default__', refresh=True):
        now_time = time.time()
        if key in cls.time_dict:
            if now_time - cls.time_dict[key] > interval:
                re_bool = True
            else:
                re_bool = False
        else:
            re_bool = False
        if refresh:
            cls.time_dict[key] = time.time()
        return re_bool

    @classmethod
    def wait_until(cls, wait_second=1, key='__default__', refresh=True):
        now_time = time.time()
        if key in cls.time_dict:
            if now_time - cls.time_dict[key] <= wait_second:
                time.sleep(wait_second + cls.time_dict[key] - now_time)
        else:
            time.sleep(wait_second)
        if refresh:
            cls.time_dict[key] = time.time()

    @classmethod
    def backup(cls, to_key, from_key='__default__'):
        if to_key in cls.time_dict:
            print('backup to_key {} exists.we will cover it.'.format(to_key), file=sys.stderr)
        if from_key not in cls.time_dict:
            raise KeyError('backup from_key {} does not exist.'.format(from_key))
        cls.time_dict[to_key] = cls.time_dict[from_key]


if __name__ == '__main__':
    pass



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/8/1

"""
import sys

__all__ = [
    'import_print_error',
    'import_install_all',
]


def import_print_error():
    MoVoidImport.print_all()


def import_install_all():
    MoVoidImport.install_all()


class MoVoidImport:
    all_package = {}

    @classmethod
    def error(cls, error: ModuleNotFoundError, version=None):
        package = error.name
        error_type = getattr(error, 'type', None)
        if error_type is None:
            if package not in cls.all_package:
                if version:
                    version_text = str(version)
                else:
                    version_text = ''
                cls.all_package[package] = {
                    'bool': True,
                    'print': 'you may lack of package: {0} .'.format(package),
                    'cmd': 'pip install {0}{1}'.format(package, version_text)
                }

    @classmethod
    def version(cls, package, version=None):
        if version is not None and not package.__version__.startswith(version):
            MoVoidImport.all_package[package.__name__] = {
                'bool': True,
                'print': 'your package {0} is version {1} but we need version {2}'.format(package.__name__, package.__version__, version),
                'cmd': 'pip install --upgrade {0}=={1}'.format(package.__name__, version)
            }
        else:
            if package.__name__ not in MoVoidImport.all_package:
                MoVoidImport.all_package[package.__name__] = {
                    'bool': False
                }

    @classmethod
    def print_all(cls):
        tell = False
        for i, v in cls.all_package.items():
            if v['bool']:
                print(v['print'], file=sys.stderr)
                tell = True
        if tell:
            print('you can run "import_install_all()" in python to install all wrong package.', file=sys.stderr)

    @classmethod
    def install_all(cls):
        import subprocess
        for i, v in cls.all_package.items():
            if v['bool']:
                print('now installing {}...'.format(i))
                subprocess.run(v['cmd'])
                print('{} install finished.'.format(i))


if __name__ == '__main__':
    pass



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/24

"""
import json

import openpyxl

a = openpyxl.Workbook()
# b = a.active
# a.remove(b)
s = a.active
with open('C:\\Users\\movoid\\Documents\\WeChat Files\\wxid_zrhiq7za1bhf22\\FileStorage\\File\\2023-07\\log_analysed.json', mode='r') as file:
    all_dict = json.load(file)['02']
print(all_dict)
index = 0
for i, v in all_dict.items():
    # s = a.create_sheet(i)

    s.cell(index * 3 + 1, 1, 'Micro Site')
    s.cell(index * 3 + 2, 1, i)
    s.cell(index * 3 + 1, 2, 'total time')
    s.cell(index * 3 + 2, 2, sum([_[-1] for _ in v]))
    s.cell(index * 3 + 1, 3, 'count')
    s.cell(index * 3 + 2, 3, len(v))
    for j, w in enumerate(v):
        s.cell(index * 3 + 1, j + 4, w[0])
        s.cell(index * 3 + 2, j + 4, w[-1])
    index += 1

a.save('log_one.xlsx')

if __name__ == '__main__':
    pass



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/24

"""
__all__ = [
    'file_write_excel_dict_list_list',
    'file_write_excel_dict_dict_list',
    'file_write_excel_dict_list_dict',
    'file_write_excel_dict_dict_dict',
    'file_read_excel_dict_list_list',
    'file_read_excel_dict_dict_list',
    'file_read_excel_dict_list_dict',
    'file_read_excel_dict_dict_dict',
]

from ..mv_import import MoVoidImport

try:
    import xlrd
except ModuleNotFoundError as err:
    MoVoidImport.error(err, '1.2.0')
else:
    MoVoidImport.version(xlrd, '1.2.0')
try:
    import xlwt
except ModuleNotFoundError as err:
    MoVoidImport.error(err)
else:
    MoVoidImport.version(xlwt)


def file_write_excel_dict_list_list(path: str, input_dict: dict):
    wb = xlwt.Workbook()
    for i, v in input_dict.items():
        ws = wb.add_sheet(i)
        for j, w in enumerate(v):
            for k, x in enumerate(w):
                ws.write(j, k, x)
    wb.save(path)


def file_write_excel_dict_dict_list(path: str, input_dict: dict):
    wb = xlwt.Workbook()
    for i, v in input_dict.items():
        ws = wb.add_sheet(i)
        j_index = 0
        for j, w in v.items():
            ws.write(j_index, 0, j)
            for k, x in enumerate(w):
                ws.write(j_index, k + 1, x)
            j_index += 1
    wb.save(path)


def file_write_excel_dict_list_dict(path: str, input_dict: dict, default=''):
    wb = xlwt.Workbook()
    for i, v in input_dict.items():
        ws = wb.add_sheet(i)
        k_list = []
        for j, w in enumerate(v):
            for k, x in w.items():
                if k not in k_list:
                    k_list.append(k)
                k_index = k_list.index(k)
                ws.write(j + 1, k_index, x)
        for j, w in enumerate(v):
            for k, x in enumerate(k_list):
                if x not in w:
                    ws.write(j + 1, k, default)
        for k, x in enumerate(k_list):
            ws.write(0, k, x)
    wb.save(path)


def file_write_excel_dict_dict_dict(path: str, input_dict: dict, default=''):
    wb = xlwt.Workbook()
    for i, v in input_dict.items():
        ws = wb.add_sheet(i)
        j_index = 0
        k_list = []
        for j, w in v.items():
            for k, x in w.items():
                if k not in k_list:
                    k_list.append(k)
                k_index = k_list.index(k)
                ws.write(j_index + 1, k_index + 1, x)
            ws.write(j_index + 1, 0, j)
            j_index += 1
        j_index = 0
        for j, w in v.items():
            for k, x in enumerate(k_list):
                if x not in w:
                    ws.write(j_index + 1, k + 1, default)
            j_index += 1
        ws.write(0, 0, i)
        for k, x in enumerate(k_list):
            ws.write(0, k + 1, x)
    wb.save(path)


def file_read_excel_dict_list_list(path: str):
    re_dict = {}
    wb = xlrd.open_workbook(path)
    wss = wb.sheets()
    for ws in wss:
        ws_name = ws.name
        re_dict[ws_name] = ws._cell_values
    return re_dict


def file_read_excel_dict_dict_list(path: str):
    re_dict = {}
    wb = xlrd.open_workbook(path)
    wss = wb.sheets()
    for ws in wss:
        ws_name = ws.name
        re_dict[ws_name] = {}
        for rows in ws._cell_values:
            re_dict[ws_name][rows[0]] = rows[1:]
    return re_dict


def file_read_excel_dict_list_dict(path: str):
    re_dict = {}
    wb = xlrd.open_workbook(path)
    wss = wb.sheets()
    for ws in wss:
        ws_name = ws.name
        re_dict[ws_name] = []
        keys = ws._cell_values[0]
        for rows in ws._cell_values[1:]:
            temp_dict = {}
            for index, cell in enumerate(rows):
                temp_dict[keys[index]] = cell
            re_dict[ws_name].append(temp_dict)
    return re_dict


def file_read_excel_dict_dict_dict(path: str):
    re_dict = {}
    wb = xlrd.open_workbook(path)
    wss = wb.sheets()
    for ws in wss:
        ws_name = ws.name
        re_dict[ws_name] = {}
        keys = ws._cell_values[0]
        for rows in ws._cell_values[1:]:
            row_key = rows[0]
            re_dict[ws_name][row_key] = {}
            for index, cell in enumerate(rows[1:]):
                re_dict[ws_name][row_key][keys[index + 1]] = cell
    return re_dict


if __name__ == '__main__':
    pass



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/27

"""

from .movoid_log import *
from .movoid_time import *
from .movoid_config import *
from .movoid_file import *
from .movoid_copy import *
from .mv_import import *

import_print_error()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/22

"""


if __name__ == '__main__':
    from movoid_log import *

    log_init('config.ini')
    for i in range(100):
        log_print('right')
